# -*- coding: utf-8 -*-

import datetime
import time

import pytz
import dateutil
import dateutil.parser
import selenium

from amazon_reports_downloader.currency_mapping import CurrencyMapping
from amazon_reports_downloader import (
    logger, get_shared_driver, MARKETPLACE_MAPPING, DEBUG
)
from amazon_reports_downloader.helpers import SellerLoginHelper
from amazon_reports_downloader.operation_recorder import OperationRecorder
from amazon_reports_downloader.payments_manager import PaymentsManager
from amazon_reports_downloader.transfer_manager import TransferManager
from amazon_reports_downloader.disbursements_manager import DisbursementsManager
from amazon_reports_downloader.exchange_rate import ExchangeRate

from amazon_reports_downloader.utils import close_web_driver
from amazon_reports_downloader.lib.google.sheet_api import SheetAPI
from amazon_reports_downloader.lib.telegram_bot import TelegramBot
from amazon_reports_downloader.lib.misc import get_chrome_path
from amazon_reports_downloader.lib.utils.puppeteer_utils import PuppeteerUtils
from pyppeteer.errors import TimeoutError


sheet_api = SheetAPI()
chicagoTz = pytz.timezone("America/Chicago") 

class DisburseTaskV2():
    def __init__(self, seller_id, marketplace, email, password, seller_profit_domain,
        rates, min_disburse_amount, sheet_id, code=None, timezone=pytz.timezone("America/Chicago"),
    ):
        self.code = code
        self.seller_id = seller_id
        self.marketplace = marketplace
        self.email = email
        self.password = password
        self.rates = rates
        self.min_disburse_amount = min_disburse_amount
        self.operation_recorder = OperationRecorder()
        self.disburse_date_format = '%Y-%m-%d'
        self.reqest_transfer_url_template = 'https://{}/payments/dashboard/index.html'
        self.disbursements_url_template = 'https://{}/payments/allstatements/index.html?ref_=xx_disbmt_ttab_dash'
        self.transfered_alerts = [
            "You cannot disburse more than once in a twenty-four hour period",
            "You cannot disburse if your available balance is zero or less",
            "You can only disburse if your available balance is above the minimum",
        ]
        self.stop_statuses = [
            'REQUEST_TRANSFER_UNAVAILABLE', 'REQUEST_TRANSFER_DISABLED',
            'INSUFFICIENT_INSTANT_TRANSFER_BALANCE', 'ALREADY_DISBURSED',
            'TRANSFER_UNAVAILABLE'
        ]
        self.save_statuses = [
            'REQUEST_TRANSFER_UNAVAILABLE', 'REQUEST_TRANSFER_DISABLED',
            'INSUFFICIENT_INSTANT_TRANSFER_BALANCE', 'ALREADY_DISBURSED',
            'TRANSFER_UNAVAILABLE', 'TRANSFER_SUCCEED'
        ]
        self.ignore_statuses = ['REQUEST_TRANSFER_FAILURE', 'TRANSFER_FAILED']
        self.timezone = timezone
        self.sheet_id = sheet_id
        self.bot = TelegramBot(api_token='6140656516:AAFRUeY4muuiIQ8uYndAB_EYUnnMescLWKU')
        self.group_chat_id = '-1001530757239'
        self.pu = PuppeteerUtils(marketplace)


    async def run(self):
        # Get last disburse record and check whether disbursed in one day
        now = datetime.datetime.now(self.timezone)
        hour_delta = datetime.timedelta(hours=1)

        records = self.get_latest_disburse_records_local(
            self.seller_id, self.marketplace, self.code
        )

        currency = CurrencyMapping.get_currency(self.marketplace)
        if currency is None or self.marketplace.lower() not in MARKETPLACE_MAPPING:
            message = '[UnsupportedMarketplace] {}'.format(self.marketplace)
            logger.error(message)

            raise ValueError(message)

        rate = ExchangeRate.get_exchange_rate('USD', currency)
        if not rate:
            rate = self.rates[currency]

        uploaded = False
        filtered_records = []
        login_records_cnt = 0
        for record_time, record in records:
            if record.get('status', '') == 'LOGIN_REQUIRED':
                login_records_cnt += 1

            if not record.get('disbursed', False):
                continue

            if 'disburse_date' not in record or not record['disburse_date']:
                continue

            disburse_date = dateutil.parser.parse(record['disburse_date'])
            if not self.is_in_one_day(disburse_date):
                continue

            filtered_records.append((record_time, record))

            ru = record.get('uploaded', False)
            if ru:
                uploaded = True

        if uploaded:
            try:
                logger.info(
                    '[AccountDisburseAlreadyUploaded] Account: {}-{}'.format(self.code, self.marketplace))
            except:
                pass
            return

        # Try to disburse only 7 times per day
        # TODO: Write to sheet
        if len(filtered_records) > 7 or login_records_cnt > 7:
            try:
                logger.info(
                    '[AccountDisburseRetryExcceeded] Account: {}-{}'.format(self.code, self.marketplace)
                )
                try:
                    self.bot.send_message(
                        self.group_chat_id,
                        "[DisburseMaxRetryExcceeded] RetryTimes: {}, SellerID: {}, Marketplace: {}, Code: {}\n".format(
                            len(filtered_records), self.seller_id, self.marketplace, self.code
                        )
                    )
                except:
                    pass
            except:
                pass
            return

        is_disburse_required = True
        is_upload_required = True
        last_disburse_date = None
        last_record_time = None
        last_record = None

        if filtered_records:
            last_record_time, last_record = filtered_records[0]

        if last_record:
            if 'disburse_date' in last_record and last_record['disburse_date']:
                try:
                    last_disburse_date = datetime.datetime.strptime(
                        last_record['disburse_date'], '%Y-%m-%dT%H:%M:%S%z')
                except:
                    last_disburse_date = last_record_time
            else:
                last_disburse_date = last_record_time

            if last_disburse_date.tzinfo is None:
                LOCAL_TIMEZONE = datetime.datetime.now(datetime.timezone.utc).astimezone().tzinfo
                last_disburse_date = last_disburse_date.replace(tzinfo=LOCAL_TIMEZONE)
        
            disbursed_in_one_day = self.is_in_one_day(last_disburse_date, now)
            if disbursed_in_one_day and last_record.get('status', '') in self.stop_statuses:
                # For compatible use only
                if 'seller_id' not in last_record:
                    last_record['seller_id'] = self.seller_id

                try:
                    logger.info('[AccountAlreadyDisbursed] Account: {}-{}'.format(self.code, self.marketplace))
                except:
                    pass

                is_disburse_required = False

                # Only upload disbursements when disbursement processed 1 hour ago
                if now - last_disburse_date > hour_delta:
                    is_upload_required = True
                else:
                    is_upload_required = False

        if not is_disburse_required and not is_upload_required:
            return

        if login_records_cnt >= 7:
            return

        page = None
        try:
            try:
                page = await self.get_dashboard(
                    self.seller_id, self.marketplace, self.email, self.password, self.min_disburse_amount,
                    self.code
                )
                helper = SellerLoginHelper(page, self.email, self.password, self.marketplace, self.code, sheet_api, self.sheet_id)
                if await helper.is_login_required():
                    raise Exception('Login Required!!!')
            except TimeoutError:
                pass
            except Exception as e:
                logger.exception(e)

                record = {
                    'status': 'LOGIN_REQUIRED',
                    'seller_id': self.seller_id,
                    'account_code': self.code,
                    'marketplace': self.marketplace,
                    'total_balance': '',
                    'unavailable_balance': '',
                    'instant_transfer_balance': '',
                    'request_transfer_available': '',
                    'disburse_date': now,
                    'message': 'Login Required!!!'
                }

                # TODO: Record login failure
                local_record = dict()
                local_record.update(record)
                local_record['disburse_date'] = record['disburse_date'].strftime('%Y-%m-%dT%H:%M:%S%z')
                self.operation_recorder.record('disburse', local_record)

                self.save_disburse(record)

                if str(e).find('Login Required') != -1:
                    message = "[LoginRequired] SellerID: {}, Marketplace: {}, Code: {}\n".format(self.seller_id, self.marketplace, self.code)
                else:
                    message = "[Error] Could not access sellercentral, bug fix is needed! SellerID: {}, Marketplace: {}, Code: {}\n".format(self.seller_id, self.marketplace, self.code)
                try:
                    self.bot.send_message(
                        self.group_chat_id, message)
                except:
                    pass

                self.pu.browser().close()
                return

            if page is None:
                return

            try:
                if is_disburse_required:
                    record = None
                    record = await self.disburse(
                        self.seller_id, self.marketplace, self.email, self.password,
                        self.min_disburse_amount, rate, self.code)

                    if record is None:
                        pass
                    elif record.get('total_balance', 0) == 0 and \
                        record.get('unavailable_balance', 0) == 0 and \
                        record.get('instant_transfer_balance', 0) == 0:
                        pass
                    else:
                        try:
                            logger.info('[DisburseProcessed] %s', record)
                        except:
                            pass

                        if record.get('status', '') in self.save_statuses:
                            self.operation_recorder.record('disburse', record)
                        else:
                            try:
                                logger.info(record)
                            except:
                                pass

                    # Don't check disbursements when just disbursed
                    return

                if not is_upload_required:
                    return

                # Upload payout records
                payout_in_one_day_found = False
                payouts, present_settlement = await self.get_most_recent_payouts(
                    self.seller_id, self.marketplace, self.email, self.password,
                    self.min_disburse_amount, rate, self.code
                )
                allowed_time = datetime.timedelta(days=1, hours=12)
                if len(payouts) == 0 and present_settlement:
                    self.save_disburse(record)
                    record['uploaded'] = True
                    self.operation_recorder.record('disburse', record)
                    return

                for payout in payouts:
                    if not payout:
                        continue

                    if payout['status'] == 'inactive':
                        # Just disbursed, payout does not come out yet
                        # Do nothing, try to obtain information next run
                        payout_in_one_day_found = True
                        break

                    if not payout.get('scheduled_payout_time', None):
                        # Ignore abnormal case
                        continue

                    payout_time = payout['scheduled_payout_time'].replace(tzinfo=datetime.timezone.utc)
                    if (now - payout_time) < allowed_time:
                        available_record = None
                        for _, record in filtered_records:
                            if record.get('status', '') in self.stop_statuses:
                                available_record = record
                                break

                        if available_record:
                            available_record.update({
                                'status': 'ALREADY_DISBURSED',
                                'amount': payout['payout_amount']
                            })
                        else:
                            available_record = {
                                'status': 'ALREADY_DISBURSED',
                                'disbursed': True,
                                'seller_id': self.seller_id,
                                'marketplace': self.marketplace.lower(),
                                'account_code': self.code,
                                'amount': payout['payout_amount'],
                                'total_balance': ''
                            }

                        if payout.get('inaccurate_payout_time', True):
                            if 'disburse_date' not in available_record:
                                record['disburse_date'] = now.strftime('%Y-%m-%dT%H:%M:%S%z')
                        else:
                            available_record['disburse_date'] = payout['scheduled_payout_time'].replace(
                                tzinfo=self.timezone).strftime('%Y-%m-%dT%H:%M:%S%z')

                        self.save_disburse(record)

                        payout_in_one_day_found = True
                        record['uploaded'] = True
                        self.operation_recorder.record('disburse', record)

                        break

                if not is_disburse_required and not payout_in_one_day_found and last_record:
                    try:
                        last_record['action'] = 'correction'
                        self.save_disburse(last_record)

                        last_record['uploaded'] = True
                        self.operation_recorder.record('disburse', last_record)
                    except Exception as e:
                        logger.exception(e)
            except Exception as e:
                logger.exception(e)
            finally:
                try:
                    self.pu.browser().close()
                except Exception as e:
                    pass
        finally:
            try:
                self.pu.browser().close()
            except Exception as e:
                pass


    async def disburse(self, seller_id, marketplace, email, password, min_disburse_amount, rate, code=None, retry_in_visible_area=False):
        currency = CurrencyMapping.get_currency(marketplace)
        if currency is None or marketplace.lower() not in MARKETPLACE_MAPPING:
            message = '[UnsupportedMarketplace] {}'.format(marketplace)
            logger.error(message)

            raise ValueError(message)

        sc_host = MARKETPLACE_MAPPING.get(marketplace.lower())['sellercentral']
        request_transfer_url = self.reqest_transfer_url_template.format(sc_host)
        page = await self.pu.fetch_url_v2(request_transfer_url)
        if not page:
            return

        payments_manager = PaymentsManager(page, marketplace)
        transfer_manager = TransferManager(page)

        retries = 3
        total_balance = None
        while retries > 0:
            try:
                total_balance = await payments_manager.get_total_balance()
            except Exception as e:
                logger.exception(e)

            if total_balance is not None:
                break

            retries -= 1
            await page.reload()
            time.sleep(3)

            continue

        if total_balance is None:
            return

        try:
            unavailable_balance = await payments_manager.get_unavailable_balance()
        except:
            unavailable_balance = None

            message = 'Could not extract unavailable balance!'
            try:
                logger.info(
                    'Could not extract total balance - SellerID: %s, Marketplace: %s',
                    seller_id or code, marketplace)
            except:
                pass

        if unavailable_balance is None:
            unavailable_balance = 0

        total_balance_in_usd = round(total_balance / rate, 2)
        if unavailable_balance:
            unavailable_balance_in_usd = round(unavailable_balance / rate, 2)
        else:
            unavailable_balance_in_usd = 0
        disburse_date = datetime.datetime.now(self.timezone)
        disburse_date_str = disburse_date.strftime('%Y-%m-%dT%H:%M:%S%z')
        record = {
            'seller_id': seller_id,
            'account_code': code,
            'marketplace': marketplace,
            'disburse_date': disburse_date_str,
            'total_balance': total_balance_in_usd,
            'unavailable_balance': unavailable_balance_in_usd,
            'request_transfer_available': True
        }

        instant_transfer_balance = await payments_manager.get_instant_transfer_balance()
        if instant_transfer_balance:
            record['instant_transfer_balance'] = round(instant_transfer_balance / rate, 2)
        else:
            instant_transfer_balance = 0
            record['instant_transfer_balance'] = 0

        disburse_button = await payments_manager.get_disburse_button()
        if not disburse_button:
            # Handle disburse unavailable
            try:
                logger.info(
                    '[DisburseUnavailable] SellerID: %s, Marketplace: %s', seller_id, marketplace)
            except:
                pass

            record.update({
                'status': 'REQUEST_TRANSFER_UNAVAILABLE',
                'amount': 0,
                'disbursed': True,
                'request_transfer_available': False,
                'message': 'Request transfer button is not found!'
            })

            return record

        if instant_transfer_balance <= min_disburse_amount:
            # Instant transfer balance is below minimum, try again later
            try:
                logger.info(
                    '[InsufficientInstantTransferBalance] SellerID: %s, Marketplace: %s',
                    seller_id or code, marketplace)
            except:
                pass

            record.update({
                'status': 'INSUFFICIENT_INSTANT_TRANSFER_BALANCE',
                'amount': 0,
                'disbursed': True,
                'message': 'InsufficientInstantTransferBalance'
            })

            return record

        # disburse_button_disabled = await payments_manager.is_disburse_button_disabled(
        #     disburse_button
        # )

        disburse_button_disabled = True if disburse_button == 'disabled' else False

        if disburse_button_disabled:
            try:
                logger.info(
                    '[AccountDisburseButtonDisabled] SellerID: %s, Marketplace: %s',
                    seller_id or id, marketplace)
            except:
                pass

            record.update({
                'status': 'REQUEST_TRANSFER_DISABLED',
                'amount': 0,
                'disbursed': True,
                'message': 'Disburse button disabled!'
            })

            return record

        result = await payments_manager.trigger_disburse()
        if not result:
            try:
                logger.warning(
                    '[TriggerDisburseError] SellerID: %s, Marketplace: %s',
                    seller_id or code, marketplace)
            except:
                pass
            record.update({
                'status': 'REQUEST_TRANSFER_FAILURE',
                'amount': 0,
                'disbursed': False,
                'message': 'Request transfer failure!'
            })

            return record

        message = 'Request disbursement succeed!'
        if await transfer_manager.is_transfer_available():
            transfer_amount = await transfer_manager.get_transfer_amount(marketplace)
            transfer_amount_in_usd = round(transfer_amount / rate, 2)

            result = await transfer_manager.trigger_transfer()
            if result and await transfer_manager.is_transfer_success():
                record.update({
                    'status': 'TRANSFER_SUCCEED',
                    'amount': transfer_amount_in_usd,
                    'disbursed': True
                })
            else:
                if result:
                    if await transfer_manager.has_transfer_alert():
                        message = await transfer_manager.get_transfer_alert()
                        already_disbursed = any(
                            [message.find(alert) != -1 for alert in self.transfered_alerts])
                        if already_disbursed:
                            try:
                                logger.info(
                                    '[AlreadyDisbursed] SellerID: %s, Marketplace: %s',
                                    seller_id or code, marketplace)
                            except:
                                pass

                            record.update({
                                'status': 'ALREADY_DISBURSED',
                                'amount': 0,
                                'disbursed': True,
                                'message': 'ALREADY_DISBURSED'
                            })
                    else:
                        record.update({
                            'status': 'TRANSFER_UNKNOWN',
                            'amount': 0,
                            'disbursed': False,
                            'message': "Unknown error. " + \
                                "Disbursement is requested and there is no alert."
                        })
                else:
                    record.update({
                        'status': 'TRANSFER_UNKNOWN',
                        'amount': 0,
                        'disbursed': False,
                        'message': 'Request disbursement failed!'
                    })
        else:
            transfer_amount = 0
            record['amount'] = 0
            if await transfer_manager.has_transfer_alert():
                message = await transfer_manager.get_transfer_alert()
                already_disbursed = any(
                    [message.find(alert) != -1 for alert in self.transfered_alerts])
                if already_disbursed:
                    try:
                        logger.info(
                            '[AlreadyDisbursed] SellerID: %s, Marketplace: %s',
                            seller_id or code, marketplace)
                    except:
                        pass

                    record.update({
                        'status': 'ALREADY_DISBURSED',
                        'disbursed': True,
                        'message': 'ALREADY_DISBURSED'
                    })
                else:
                    record.update({
                        'status': 'TRANSFER_UNAVAILABLE',
                        'disbursed': True,
                        'message': message
                    })
            else:
                record.update({
                    'status': 'TRANSFER_UNAVAILABLE',
                    'disbursed': True,
                    'message': 'Request disbursement is unavailable!'
                })

        await transfer_manager.return_to_summary()

        return record

    async def get_most_recent_payouts(
        self, seller_id, marketplace, email, password, min_disburse_amount,
        rate, code=None, retry_in_visible_area=False
    ):

        sc_host = MARKETPLACE_MAPPING.get(marketplace.lower())['sellercentral']
        disbursements_url = self.disbursements_url_template.format(sc_host)

        page = await self.pu.fetch_url_v2(disbursements_url)
        disbursements_manager = DisbursementsManager(page, marketplace, rate)
        present_settlement = None
        disbursements = []
        retries = 3
        while retries > 0:
            try:
                disbursements = await disbursements_manager.get_disbursements()
                present_settlement = present_settlement or await disbursements_manager.get_present_settlement()
            except Exception as e:
                logger.exception(e)

            if disbursements:
                break

            retries -= 1

            await page.reload()
            time.sleep(3)

        return disbursements, present_settlement

    async def get_dashboard(self, seller_id, marketplace, email, password, min_disburse_amount, code=None, retry_in_visible_area=False):
        sc_host = MARKETPLACE_MAPPING.get(marketplace.lower())['sellercentral']

        seller_central_url = 'https://{}/home'.format(sc_host)

        page = await self.pu.fetch_url_v2(seller_central_url)
        helper = SellerLoginHelper(page, email, password, marketplace, code, sheet_api)

        is_logged = await helper.is_login_required()
        while is_logged:
            logger.info('Login required! Trying to login...')

            await helper.login()
            await helper.send_otp()
            await helper.check_opt()

            wait_time = 180
            while wait_time > 0:
                wait_time -= 1
                logger.debug('Waiting for login...')
                if await helper.is_login_required():
                    time.sleep(2)
                else:
                    break

            if wait_time <= 0:
                message = '[LoginSellerCentralFailed] SellerID: {}, Marketplace: {}'.format(
                    seller_id, marketplace
                )
                logger.error(message)
                return page

            time.sleep(7)

            is_logged = await helper.is_login_required()

        if await helper.is_login_required():
            message = '[LoginSellerCentralFailed] SellerID: {}, Marketplace: {}'.format(
                seller_id, marketplace
            )
            logger.error(message)
            return page

        # if DEBUG or retry_in_visible_area:
        #     await page.setViewport({'x': 0, 'y': 0})
        # else:
        #     await page.setViewport({'x': origin_x, 'y': origin_y})

        logger.info('begin to pick marketplace')
        await helper.pick_marketplace()
        logger.info('Picked marketplace!')

        return page

    def check_exist_in_template(self, record):
        key = record['account_code'] + '-' + record['marketplace']
        sheet_name = "template"
        worksheet = sheet_api.create_new_sheet_if_not_existed(self.sheet_id, sheet_name=sheet_name)
        data = {0: '%s-%s' % (record['account_code'], record['marketplace'])}
        update_ignores = [0]
        sheet_api.append_or_insert_row(worksheet, key=key, data=data, update_ignores=update_ignores, insertOnly=True)

    def save_disburse(self, record):
        now = datetime.datetime.now(self.timezone)
        if not record.get('disburse_date', ''):
            record['disburse_date'] = now.strftime('%Y-%m-%dT%H:%M:%S%z')

        amount = record.get('amount', record.get('disbursed_amount', 0))
        message = record.get('status', record.get('message', 'No message available!'))
        if not message:
            message = 'No message available!'
        formatted_disburse = {
            'seller_id': record['seller_id'],
            'marketplace': record['marketplace'],
            'amount': amount,
            'total_balance': record['total_balance'],
            'unavailable_balance': record['unavailable_balance'],
            'instant_transfer_balance': record.get('instant_transfer_balance', amount),
            'request_transfer_available': record.get('request_transfer_available', True),
            'disburse_date': record['disburse_date'],
            'message': message
        }

        result = None

        sheet_id = self.sheet_id
        # disburse_date = datetime.datetime.strptime(, '%Y-%m-%dT%H:%M:%S%z')
        if isinstance(record['disburse_date'], str):
            disburse_date = dateutil.parser.parse(record['disburse_date'])
            disburse_date_s = record['disburse_date']
        else:
            disburse_date = record['disburse_date']
            disburse_date_s = record['disburse_date'].strftime('%Y-%m-%dT%H:%M:%S%z')
        sheet_name = disburse_date.strftime('%m/%d')
        sheets = list()
        self.check_exist_in_template(record)
        if len(sheets) == 0:
            try:
                worksheet = sheet_api.create_new_sheet_if_not_existed(sheet_id, sheet_name=sheet_name)
                sheets.append(worksheet)
            except Exception as e:
                logger.exception(e)

        if len(sheets) > 0:
            if record.get('status', '') == 'LOGIN_REQUIRED':
                data = {8: disburse_date_s, 9: message}
            elif record.get('action', '') == 'correction':
                data = {
                        4: record['total_balance'],
                        5: record['unavailable_balance'],
                        6: record.get('instant_transfer_balance', amount),
                        7: record.get('request_transfer_available', True),
                        8: disburse_date_s,
                        9: message
                        }
            else:
                data = {0: '%s-%s' % (record['account_code'], record['marketplace']),
                        1: record['account_code'],
                        2: record['marketplace'],
                        3: amount,
                        4: record['total_balance'],
                        5: record['unavailable_balance'],
                        6: record.get('instant_transfer_balance', amount),
                        7: record.get('request_transfer_available', True),
                        8: disburse_date_s,
                        9: message
                        }
            key = record['account_code'] + '-' + record['marketplace']
            update_ignores = [0]
            for sheet in sheets:
                sheet_api.append_or_insert_row(sheet, key=key, data=data, update_ignores=update_ignores, insertOnly=False, update_only=True)

    def get_latest_disburse_records_local(self, seller_id, marketplace, code=None):
        return self.operation_recorder.get_last_records(
            'disburse', 1,
            lambda record_time, record: self.is_disburse_of_account(record_time, record, seller_id, marketplace, code))

    def get_latest_disburse_record_local(self, seller_id, marketplace, code=None):
        return self.operation_recorder.get_last_record(
            'disburse',
            lambda record_time, record: self.is_disburse_of_account(record_time, record, seller_id, marketplace, code))

    def is_disburse_of_account(self, record_time, record, seller_id, marketplace, code=None):
        marketplace = marketplace.lower()

        return record.get('disbursed', False) and \
            (record.get('seller_id', '') == seller_id or record.get('account_code', '') == code) and \
            record.get('marketplace', '').lower() == marketplace

    def is_in_one_day(self, d1, d2 = None):
        if d2 is None:
            d2 = datetime.datetime.now(self.timezone)
        time_passed = d2 - d1
        return (time_passed < datetime.timedelta(days=1, minutes=15))
