# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import re

from amazon_reports_downloader.utils import extract_balance_amount
import asyncio
from pyppeteer import launch
from pyppeteer.errors import PageError, TimeoutError

class TransferManager:
    def __init__(self, page):
        self.page = page
        self.selectors = {
            'xpathes': {
                'transfer_amount': '//*[@id="currentBalanceValue" or @id="availableBalanceValue"]/span',
                'transfer_amount_v2': '//div[@class="settlement-amount-balance"]',
                'request_transfer_button': '//*[@id="request_transfer_button" or @id="disburseToBankAccountButton" or @label="Request Disbursement"]',
                'request_transfer_button_v2': '//div[@class="disburse-now-button"]/kat-button',
                'return_to_summary_button': '//*[@id="go_back_button" or @id="goBackButton"]',
                'return_to_summary_button_v2': '//div[@class="return-to-account-buttons"]/kat-button'
            }
        }

    async def get_transfer_amount(self, marketplace):
        current_balance = None
        try:
            current_balance_elem = await self.page.waitForXPath(self.selectors['xpathes']['transfer_amount'], timeout=3000)
            current_balance_str = await current_balance_elem.getProperty('textContent')
            current_balance_str = await current_balance_str.jsonValue()
            current_balance_str = current_balance_str.strip()
            current_balance = extract_balance_amount(marketplace, current_balance_str)
        except Exception:
            try:
                current_balance_elem_v2 = await self.page.waitForXPath(self.selectors['xpathes']['transfer_amount_v2'], timeout=3000)
                current_balance_str_v2 = await current_balance_elem_v2.getProperty('textContent')
                current_balance_str_v2 = await current_balance_str_v2.jsonValue()
                current_balance_str_v2 = current_balance_str_v2.strip()
                current_balance = extract_balance_amount(marketplace, current_balance_str_v2)
            except Exception as e:
                raise e

        return current_balance


    async def is_transfer_available(self):
        try:
            await self.page.waitForNavigation({'waitUntil': 'domcontentloaded'}),
            await self.page.waitForXPath(self.selectors['xpathes']['request_transfer_button'], timeout=3000)
            return True
        except TimeoutError:
            try:
                await self.page.waitForXPath(self.selectors['xpathes']['request_transfer_button_v2'], timeout=3000)
                return True
            except TimeoutError:
                return False

    async def is_transfer_success(self):
        try:
            await self.page.waitForXPath('//*[contains(text(), "Successfully")]', timeout=3000)
            return True
        except TimeoutError:
            try:
                await self.page.waitForXPath('//kat-alert[@variant="success"]', timeout=3000)
                return True
            except TimeoutError:
                return False

    async def has_transfer_alert(self):
        try:
            await self.page.waitForXPath('//div[@id="DisburseNowDetails"]/div[contains(@class, "infoMessages")]', timeout=3000)
            return True
        except TimeoutError:
            try:
                await self.page.waitForXPath('//*[contains(@class, "ineligibility-alert")]', timeout=3000)
                return True
            except TimeoutError:
                return False

    async def get_transfer_alert(self):
        try:
            alert_elem = await self.page.waitForXPath('//div[@id="DisburseNowDetails"]/div[contains(@class, "infoMessages")]//ul', timeout=3000)
            alert_message = await alert_elem.getProperty('textContent')
            alert_message = await alert_message.jsonValue()
            return alert_message.strip()
        except TimeoutError:
            try:
                alert_elem = await self.page.waitForXPath('//*[contains(@class, "ineligibility-alert")]', timeout=3000)
                transfer_alert = not await self.is_transfer_success()
                alert_message = await alert_elem.getProperty('textContent')
                alert_message = await alert_message.jsonValue()
                return alert_message.strip() if transfer_alert else ''
            except TimeoutError:
                return ''

    async def trigger_transfer(self):
        try:
            request_transfer_button_elem = await self.page.waitForXPath(self.selectors['xpathes']['request_transfer_button'], timeout=3000)
            await request_transfer_button_elem.click()
            return True
        except TimeoutError:
            try:
                request_transfer_button_elem = await self.page.waitForXPath(self.selectors['xpathes']['request_transfer_button_v2'], timeout=3000)
                await request_transfer_button_elem.click()
                return True
            except TimeoutError:
                return False

    async def return_to_summary(self):
        try:
            return_to_summary_elem = await self.page.waitForXPath(self.selectors['xpathes']['return_to_summary_button'], timeout=3000)
            await return_to_summary_elem.click()
            return True
        except TimeoutError:
            try:
                return_to_summary_elem = await self.page.waitForXPath(self.selectors['xpathes']['return_to_summary_button_v2'], timeout=3000)
                await return_to_summary_elem.click()
                return True
            except TimeoutError:
                return False

